#!/usr/bin/perl -w
# +========================================================================+
# || cipux_set_value                                                      ||
# || This script sets a CipUX LDAP value to a LDAP entry. It uses the     ||
# || ldapmodify command to perfom the task.                               ||
# ||                                                                      ||
# ||                                                                      ||
# || Copyright: (c) 1999 - 2005  by Christian Kuelker all Rights reserved!||
# || Licence:   GPLv2  (GPL version 2 or later)                           ||
# ||                                                                      ||
# || Revision:  2005-08-03                                                ||
# ||                                                                      ||
# +========================================================================+


=pod

=head1 NAME

cipux_set_value - Version <[CIBOT::VERSION]>  2003-03-22 CipUX

=head1 SYNOPSIS

	cipux_set_value [OPTIONS] -e <ENTRY> -v <VALUE>

	cipux_set_value -h

	Options: (exact only one is mandatory)
	(01) -u <UCN>  :set user value                   (modify)
	(02) -c <CCN>  :set course value                 (modify)
	(03) -m <MCN>  :set machine value                (modify)
	(04) -r <RCN>  :set room value                   (modify)
        (05) -t <TCN>  :set hardware type value          (modify)
        (06) -i <ICN>  :set image value                  (modify)
        (07) -s <SCN>  :set image slot value             (modify)
        (08) -y <YCN>  :set admin value                  (modify)
        (09) -x <XCN>  :set configuration value          (modify)
	(10) -U        :set value for all users          (modify)
	(11) -C        :set value for all courses        (modify)
	(12) -M        :set value for all machines       (modify)
	(13) -R        :set value for all rooms          (modify)
        (14) -T        :set value for all hardware types (modify)
        (15) -I        :set value for all images         (modify)
        (16) -S        :set value for all image slots    (modify)
        (17) -Y        :set value for all admins         (modify)
        (18) -X        :set value for all configuration  (modify)
        Auxilliary options:
	(19) -h        :get help page
        (20) -a        :this actually adds a value (add)
        (21) -d        :EXPERIMENTAL! replace one value, or if only one value left, deletes
                        it and leave an empty attribute behind.

	UCN = U+CN = U=user + CN = canonical name
	CCN, MCN, RCN analog

=head1 ABSTRACT

This script sets a value on a LDAP server for a given entry (attribute).
It can be done in two modes (A) all nodes and (B) one node.
Nine fields can be choosen:
(1) user,
(2) course,
(3) machine,
(4) room,
(5) hardware type,
(6) image,
(7) image slot,
(8) admin and
(9) configuration

For Skolelinux there are only (1) user and (2) course possible.

Be careful with the global mode (A) it will set a value to all nodes!

If you want to add a value (example: memberUid for posixGroup) rather
than to modify, you have to add the -a option!

=head1 REQUIREMENTS

	CipUX::CiBot;
        Getopt::Std

=cut

package cipux_set_value;

# +========================================================================+
my $PART=                    "Global Part";#                               |
# +========================================================================+

my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "cipux_set_value";
my $VERSION = "<[CIBOT::VERSION]>";
my $BUILD   = "<[CIBOT::BUILD]>";

# some global VARs:
my %values=();
my @values=();
my %opt;
my $dn_search = "";
my $command = "";

&log($SCRIPT,"="x76);
&log($SCRIPT,"BEGIN LOG: $SCRIPT $VERSION <$BUILD>");
my $cml = join(' ',@ARGV);
&log($SCRIPT," $SCRIPT $cml");

# Usage of cipworx cipux modules and libraries
use strict;
use CipUX::CiBot;
use Getopt::Std;
use MIME::Base64;

&cibot_get_ldap_config();

# +========================================================================+
$PART=                       "Environment";#                               |
# +========================================================================+

# use of environment modules
my @one_mandatory_option=qw(u: c: m: r: t: i: s: y: x: U C M R T I S Y X);
my $options="adv:e:h";
foreach(@one_mandatory_option){$options.=$_}
&log($SCRIPT,"- Options: '$options'");

getopts("$options",\%opt);

if(defined($opt{h})){&usage;exit(0);}

#( only if NOT root security)
#if(!defined($opt{p})){&usage;error('0005');}#cipux_set_value: No option -p, please give it to me
if(!defined($opt{e})){&usage;error('0006');}#cipux_set_value: No option -e, please give it to me
if(!defined($opt{v})){&usage;error('0010');}#cipux_set_value: No option -v, please give it to me

# remember choosen command line options:
my %test                      = ();
my $search_node             = "";
my $one_mandatory_option_list = "";
my $filter                    = "";# 'cn=*' only for -U -C -M ...
my $capital_option            = 0;
my $searchbase                = "";
my $dn_prefix                 = "";

foreach (@one_mandatory_option){
  s/://g;# we don't need the : any more!
  if(defined($opt{$_})){
    # Example           x             cibot
      &log($SCRIPT,"- parse option '$_' with value '$opt{$_}'\n");
    $test{$_} = 1; # $test{U} = 1 if defined($opt{U});
    $search_node = "-$_"; # $search_node = "-U" if defined($opt{U});

    # searchbase
    # OLD: $searchbase=$Ldap_User_DN    if defined($opt{u}) or defined($opt{U});
    # NEW: $searchbase=$Ldap_DN_U
    chomp();
    my $string = "Ldap_DN_".uc($_); # example: Ldap_DN_U
    &log($SCRIPT,"- ldap searchbase string [$string]");
    $searchbase = $cipux{$string};
    # prefix
    # $dn_prefix="dn: ".$Ldap_DN_U_Prefix if defined($opt{U});
    $string .= "_Prefix"; # example: Ldap_DN_U_Prefix
    &log($SCRIPT,"- ldap dn prefix string [$string]");
    $dn_prefix = $cipux{$string};
    $string =~ s/_Prefix/_Search/g; # example: Ldap_DN_U_Search
    &log($SCRIPT,"- ldap dn search string [$string]");
    $dn_search = $cipux{$string};
    # 91 = Z+1, 64 = A-1   => cn=* if from A till Z
    if(ord($_) < 91 and ord($_) > 64){
      $filter=$dn_prefix.'=*' if ord($_) < 91 and ord($_) > 64; #TODO: replace uid with cn for real CipUX, uid is for Skole
     $capital_option = 1;
    }else{
      # $filter="cn=$opt{u}" if defined($opt{u});
      $filter="$dn_prefix=$opt{$_}"; #TODO: replace uid with cn for real CipUX, uid is for Skole
    }
  }
  $one_mandatory_option_list .= " -".$_;
}


  &log($SCRIPT,"- \$search_node  = '$search_node'");
  &log($SCRIPT,"- \$filter         = '$filter'");
  &log($SCRIPT,"- \$capital_option = '$capital_option'");
  &log($SCRIPT,"- \$searchbase     = '$searchbase'");
  &log($SCRIPT,"- \$dn_prefix      = '$dn_prefix'");
  &log($SCRIPT,"- \$dn_search      = '$dn_search'");

my $c=0;
foreach (sort keys %test){$c++}
# cipux_set_value: Need definitly one option from:
if($c==0){&usage;&error('0007',$one_mandatory_option_list)}
# ok
if($c==1){&log($SCRIPT,"- Got one option")}
# cipux_set_value: To many options, need only one auxiliary option
if($c>1){&usage;&error('0008',$one_mandatory_option_list)}

# (only if NOT root security)
#my $ldap_password=$opt{p};
my $entry        =$opt{e};
my $value        =$opt{v};

# +========================================================================+
$PART=                "Read password";#                                    |
# +========================================================================+

# not in use now:
# If there is no password given through the environemnt
# Wenn kein Passwort als Zeilenschalter bergeben wurde:
#if(!defined($opt{p})){
#  print "Geben sie LDAP root Passwort ein: ";
#  print "Please give the LDAP root password: ";
#  system "stty -echo";
#  chomp($ldap_password=<STDIN>);
#  print "\n";
#  system "stty echo";
#}

# +========================================================================+
$PART=              "Prepare modify";#                                     |
# +========================================================================+

#if($DEBUG){
# foreach (@VAR){
#   s/'//g;# we don't need the ' any more!
#   &log($SCRIPT,"- $_ = '".eval("$_")."'");
# }
#}

my $node="";
$node=$opt{u} if defined($opt{u});
$node=$opt{c} if defined($opt{c});
$node=$opt{m} if defined($opt{m});
$node=$opt{r} if defined($opt{r});
$node=$opt{t} if defined($opt{t});
$node=$opt{i} if defined($opt{i});
$node=$opt{s} if defined($opt{s});
$node=$opt{y} if defined($opt{y});
$node=$opt{x} if defined($opt{x});

my $all_nodes="";
$all_nodes="-U"if defined($opt{u}) or defined($opt{U});
$all_nodes="-C"if defined($opt{c}) or defined($opt{C});
$all_nodes="-M"if defined($opt{m}) or defined($opt{M});
$all_nodes="-R"if defined($opt{r}) or defined($opt{R});
$all_nodes="-T"if defined($opt{t}) or defined($opt{T});
$all_nodes="-I"if defined($opt{i}) or defined($opt{I});
$all_nodes="-S"if defined($opt{s}) or defined($opt{S});
$all_nodes="-Y"if defined($opt{y}) or defined($opt{Y});
$all_nodes="-X"if defined($opt{x}) or defined($opt{X});

my $one_node="";
$one_node="-u"if defined($opt{u}) or defined($opt{U});
$one_node="-c"if defined($opt{c}) or defined($opt{C});
$one_node="-m"if defined($opt{m}) or defined($opt{M});
$one_node="-r"if defined($opt{r}) or defined($opt{R});
$one_node="-t"if defined($opt{t}) or defined($opt{T});
$one_node="-i"if defined($opt{i}) or defined($opt{I});
$one_node="-s"if defined($opt{s}) or defined($opt{S});
$one_node="-y"if defined($opt{y}) or defined($opt{Y});
$one_node="-x"if defined($opt{x}) or defined($opt{X});

# +========================================================================+
$PART=              "Create Modify";#                                      |
# +========================================================================+
my @uid=();
my $modify="";
if($filter =~ /=\*/){ #if =* is in filter, it supposed apply to all nodes

  &log($SCRIPT,"- [A] ALL NODES MODE");
  # ALL nodes of a given type of the LDAP (eg: all USER)

  # get all $dn_search, example uid or cn
  my $cmd="/usr/bin/cipux_get_value -e $dn_search $all_nodes |";
  &log($SCRIPT,"- CMD for all [$dn_search] cmd = [$cmd]");
  open(CMD,$cmd)
    or error('0058',$all_nodes);#cipux_set_value: Can't execute get_value.pl -p <PASSWORD> -e uid
  @uid=<CMD>;
  close(CMD);

  foreach my $line (@uid){
    chomp($line);
    &log($SCRIPT,"- LINE [$line]");# this iterates over the nodes
    my($uid,$dummy)=split("\t",$line);
    if(defined($opt{a})){
      # [1] append modus # META: write error codes
      &log($SCRIPT,"- [1] APPEND MODUS (multi nodes)");
      my $cmd = "/usr/bin/cipux_get_value -e $entry $one_node $uid";
      &log($SCRIPT,"- CMD [1]: $cmd");
      my @add=`$cmd`;
      my @multiple_values=();                #mult
      @multiple_values = split("\t",$value); #mul
      $modify.="dn: $dn_prefix=$uid,$searchbase\n"
             ."changetype: modify\n"
             ."replace: $entry\n";
      if(scalar @multiple_values > 0){
        foreach(@multiple_values){
          $modify.="$entry: $_\n";# multiple news
        }
      }else{
        $modify .= "$entry: $value\n";# one new
      }
      foreach(@add){
        chomp();
        $modify.="$entry: $_\n" if $_ ne $opt{v};# olds
      }
      $modify.="\n";
      &log($SCRIPT,"-> A1 modify = [$modify]");
    }elsif(defined($opt{d})){
      # [2] delete modus # META: write error codes
      &log($SCRIPT,"- DELETE MODUS (multi nodes)");
      my $cmd = "/usr/bin/cipux_get_value -e $entry $one_node $node $uid";
      &log($SCRIPT,"- CMD [2]: $cmd");
      my @del=`$cmd`;
      my @multiple_values=();                #mul
      @multiple_values = split("\t",$value); #mul
      $modify.="dn: $dn_prefix=$uid,$searchbase\n"
             ."changetype: modify\n"
#            ."delete: $entry\n";
             ."replace: $entry\n";
      &log($SCRIPT,"-> A2 modify =[$modify]");

      &log($SCRIPT,"- MULTIPLE [".scalar @multiple_values."]");
      if(scalar @multiple_values > 0){# del more than one
        # differenzmenge perl cookbook 4.7
        my %seen=();
        my @aonly=();
        # create lookup table
        foreach(@multiple_values){chomp();$seen{$_}=1}
        # find elements which exists in @del but NOT in @multiple_values
        foreach (@del){
          chomp();
          unless($seen{$_}){
            # is not in %seen, so add to @aonly
            push(@aonly,$_);
          }
        }
        foreach(@aonly){
          chomp();
          &log($SCRIPT,"- AONLY: $_");
          $modify.="$entry: $_\n";
        }
      }else{
        foreach(@del){
          chomp();
          $modify.="$entry: $_\n" if $_ ne $value;
        }
      }


####
#     if(scalar @multiple_values > 0){
#       foreach(@multiple_values){
#         $modify.="replace: $_\n";# multiple news
#       }
#     }else{
#       $modify .= "replace: $value\n";# one new
#     }
#     foreach(@add){
#       chomp();
#       $modify.="$entry: $_\n" if $_ ne $value;
#     }
#
###
      $modify.="\n";
      &log($SCRIPT,"-> modify =[$modify]");
    }else{
      # change modus
      &log($SCRIPT,"- [A5] CHANGE MODUS");
      $modify.="dn: $dn_prefix=$uid,$searchbase\n"
             ."changetype: modify\n"
             ."replace: $entry\n"
             ."$entry: $value\n\n";
      &log($SCRIPT,"-> [A5] modify =[$modify]");
    }
  }
}else{
  # ONE node of a given type of the LDAP (eg: one COURSE)
  &log($SCRIPT,"- [B] ONE NODE MODE");
  # Create a virtual modify file
  if(defined($opt{a})){
    # [3] append modus # META: write error codes
    &log($SCRIPT,"- APPEND MODUS (single node)");
    &log($SCRIPT,"- CMD [3]: /usr/bin/cipux_get_value -e $entry $one_node $node");
    my @add=`/usr/bin/cipux_get_value -e $entry $one_node $node`;
    my @multiple_values=();                #mul
    my %multiple_values=();                #mul saver index for -x to chose only las
    @multiple_values = split("\t",$value); #mul
    $modify.="dn: $dn_prefix=$node,$searchbase\n"
           ."changetype: modify\n"
           ."replace: $entry\n";
    if(scalar @multiple_values > 0){
      foreach(@multiple_values){
	$modify.="$entry: $_\n";# multiple news
	if(defined($opt{x}) and $entry eq "cipuxVariable"){# for cipuxVariable
	  # this will lead to delete double cipuxVariables
	  my($k,$v) = split('=',"$entry: $_",2);
	  &log($SCRIPT," - remember multiple_values key [NOT DEFINED?] value [$v]") if defined($v) ;
	  &log($SCRIPT," - remember multiple_values key [$k]           value [NOT DEFINED?]") if defined($k);
	  &log($SCRIPT," - remember multiple_values key [$k]           value [$v]") if(defined($k) and defined($v));
	  $multiple_values{$k}=$v;
	}
      }
    }else{
      $modify .= "$entry: $value\n";# one new
      if(defined($opt{x}) and $entry eq "cipuxVariable"){# for cipuxVariable
	# this will lead to delete double cipuxVariables
	my($k,$v) = split('=',"$entry: $_");
	&log($SCRIPT," - remember one_values key [$k] value [$v]");
	$multiple_values{$k}=$v;
      }
    }
    foreach(@add){
      chomp();
      if(defined($opt{x}) and $entry eq "cipuxVariable"){# for cipuxVariable
	# this will lead to delete double cipuxVariables
	my($k,$v) = split('=',"$entry: $_",2);
	&log($SCRIPT," - test for add  multiple_values key [$k] value [$v]") if(defined($k) and defined($v));
	&log($SCRIPT," - test for add  multiple_values key [??] value [$v]") if defined($v);
	&log($SCRIPT," - test for add  multiple_values key [$k] value [??]") if defined($k);
	if(defined($multiple_values{$k})){
	  &log($SCRIPT," - [$k] is defined, do not add ");
	}else{
	  &log($SCRIPT," - [$k] is NOT defined, DO add ");
	  $modify.="$entry: $_\n"  if $_ ne $opt{v};# olds
	}
	#
      }else{
	# add all olds
	$modify.="$entry: $_\n"  if $_ ne $opt{v};# olds
      }
    }
    $modify.="\n";
  }elsif(defined($opt{d})){
    # [4] delete modus # META: write error codes
    &log($SCRIPT,"- [4] DELETE MODUS (single node)");
    &log($SCRIPT,"- CMD: /usr/bin/cipux_get_value -e $entry $one_node $node");
    my @del=`/usr/bin/cipux_get_value -e $entry $one_node $node`;
    my @multiple_values=();                #mul
    @multiple_values = split("\t",$value); #mul
    $modify.="dn: $dn_prefix=$node,$searchbase\n"
           ."changetype: modify\n"
#          ."delete: $entry\n";
	   ."replace: $entry\n";

    &log($SCRIPT,"- MULTIPLE ".scalar @multiple_values."");
    #if(scalar @multiple_values > 0){
    #  foreach(@multiple_values){
    #    $modify.="replace: $_\n";# multiple news
    #  }
    #}else{
    #  $modify .= "replace: $value\n";# one new
    #}
    if(scalar @multiple_values > 0){# del more than one
      # differenzmenge perl cookbook 4.7
      my %seen=();
      my @aonly=();
      # create lookup table
      foreach(@multiple_values){chomp();$seen{$_}=1}
      # find elements which exists in @del but NOT in @multiple_values
      foreach (@del){
        chomp();
        unless($seen{$_}){
          # is not in %seen, so add to @aonly
          push(@aonly,$_);
        }
      }
      foreach(@aonly){
        chomp();
        &log($SCRIPT,"- AONLY: $_");
        $modify.="$entry: $_\n" if $_ ne $value;

      }
    }else{
      foreach(@del){
        chomp();
        $modify.="$entry: $_\n" if $_ ne $value;
      }
    }
    $modify.="\n";
    #print "MODIFY: [$modify]\n";
  }else{
     # change modus
     &log($SCRIPT,"- [B5] CHANGE MODUS");
     $modify="dn: $dn_prefix=$node,$searchbase\n"
            ."changetype: modify\n"
            ."replace: $entry\n"
            ."$entry: $value\n";
   }
}

# /usr/local/bin/ldapmodify  -rc -p 389 -h localhost -x -w SECRET
#                            -D 'uid=root,o=cipworx'
 $command="| $cipux{Ldap_Modify_Command} -rc -p $cipux{Ldap_Port}"
	    ." -h $cipux{Ldap_Host} -x $cipux{Ldap_Start_TLS} -w $cipux{Ldap_Password}"
	    ." -D $cipux{Ldap_Root_DN}";

&log($SCRIPT,"- dn_prefix           = '$dn_prefix'");
&log($SCRIPT,"- node              = '$node'");
&log($SCRIPT,"- searchbase          = '$searchbase'");
&log($SCRIPT,"- entry               = '$entry'");
&log($SCRIPT,"- value               = '$value'");
&log($SCRIPT,"- Ldap_Modify_Command = '$cipux{Ldap_Modify_Command}'");
&log($SCRIPT,"- Ldap_Port           = '$cipux{Ldap_Port}'");
&log($SCRIPT,"- Ldap_Host           = '$cipux{Ldap_Host}'");
&log($SCRIPT,"- Ldap_Start_TLS      = '$cipux{Ldap_Start_TLS}'");
#&log($SCRIPT,"- Ldap_Password       = '$cipux{Ldap_Password}'") if $DEBUG == 10;
&log($SCRIPT,"- Ldap_Root_DN        = '$cipux{Ldap_Root_DN}'");
# +========================================================================+
$PART=              "Main part";#                                          |
# +========================================================================+
#$modify = "dn: ".$modify;
&log($SCRIPT,"- CMD $command");
my $pid=open(MODIFY,"$command > /dev/null")
  or error('0011');# cipux_set_value: Can't execute LDAP modify command
&log($SCRIPT,"- MODYFIY: '$modify'");
print MODIFY $modify;
#` echo '$modify' > /tmp/mod.ldif`;
close(MODIFY);

error('0012') if !defined($pid); # cipux_set_value: Error occured while executing LDAP modify

exit(0);# exit ok


# +================================ $modify > /tmp/mod.ldif`;
close(MODIFY);

error('0012') if !defined($pid); # cipux_set_value: Error occured while executing LDAP modify

exit(0);# exit ok


# +========================================================================+
$PART=              "Subroutines";#                                        |
# +========================================================================+

=pod

=head1 FUNCTIONS

=cut

# .........................................................................
# :                               usage                                   :
# .........................................................................

=pod

=head2 usage

Prints usage message.

=cut

#============================================================|

sub usage{

print "-"x76;
print "\n";
print "($BRAND-$PROJECT-$SCRIPT-$VERSION)\n\n";
print "USAGE: \n";
print " $SCRIPT.pl [OPTIONS] -e <ENTRY> -v <VALUE>\n";
print "Options: (exact only one is mandatory)\n";
print "(01) -u <UCN>  :set user value                   (modify)\n";
print "(02) -c <CCN>  :set course value                 (modify)\n";
print "(03) -m <MCN>  :set machine value                (modify)\n";
print "(04) -r <RCN>  :set room value                   (modify)\n";
print "(05) -t <TCN>  :set hardware type value          (modify)\n";
print "(06) -i <ICN>  :set image value                  (modify)\n";
print "(07) -s <SCN>  :set image slot value             (modify)\n";
print "(08) -y <YCN>  :set admin value                  (modify)\n";
print "(09) -x <XCN>  :set configuration value          (modify)\n";
print "(10) -U        :set value for all users          (modify)\n";
print "(11) -C        :set value for all courses        (modify)\n";
print "(12) -M        :set value for all machines       (modify)\n";
print "(13) -R        :set value for all rooms          (modify)\n";
print "(14) -T        :set value for all hardware types (modify)\n";
print "(15) -I        :set value for all images         (modify)\n";
print "(16) -S        :set value for all image slots    (modify)\n";
print "(17) -Y        :set value for all admins         (modify)\n";
print "(18) -X        :set value for all configuration  (modify)\n";
print "Axilliary Options:\n";
print "(19) -h        :get this help page\n";
print "(20) -a        :this actually adds a value (add)\n";
print "(21) -d        :EXPERIMENTAL: replace a value\n";
print "\n";
print "UCN = U+CN = U=user + CN = canonical name\n";
print "...\n";
print "-"x76;
print "\n";
}

#===========================================================================
#==== Documentation ========================================================
#===========================================================================


=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

cipux_get_value

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut

